//
//  UCPClientService.h
//  UCPFramework
//
//  Created by Christian Weykopf on 02.10.09.
//  Copyright 2009 STARFACE GmbH. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "Protocols.h"

@class XMPPConnection;

/*!
    @brief    Base class for all clients.
    @details  This is the base-class for all UCP clients.
 
              Subclasses must override providedService and xmppConnection:didReceiveEvent:params:.
*/
@interface UCPClientService : NSObject  <SFClientProtocol>
{
    id _delegate;
    
    // this dictionary contains arrays of invocations
    // key is the servicename
    // every array contains dictionarys with the invocation as nonretained value and the corresponding connection as key
    NSMutableDictionary *_registeredInvocations;
}

@property (assign) id delegate;
@property (retain) NSMutableDictionary *registeredInvocations;

/*!
    @brief Designated initializer.
    @details Initializes the class ans sets the delegate.
*/
- (id) initWithDelegate: (id)delegate;

/*!
    @brief      Returns the provides Service of the client.
    @details    Subclasses must override this method.
    @return     The service as string.
*/
- (NSString *) providedService;

/*!
    @brief      Registers an invocation for a service
    @details    Subclasses must override this method.
    @param      invocation The invocation called if service name arrives.
    @param      serviceName The service name to register.
    @return     YES if service name handeled by class
 */
- (BOOL) registerInvocation: (NSInvocation *) invocation forServiceName: (NSString *)serviceName;

/*!
    @brief      Removes an invocation for a servie
    @details    Subclasses must override this method. You must specify either aTarget or aServicename or both.
    @param      aTarget The target for the service. Can be nil.
    @param      serviceName The service name to remove. Can be nil.
 */
- (void) removeInvocationForTarget: (id)aTarget serviceName: (NSString *)aServiceName;

/*!
    @brief      Called from the xmpp framework
    @param      sender Instance of the sender.
    @param      connectionEvent The event.
    @param      params EventParameters as Dictionary. Could be nil.
*/
- (void) xmppConnection: (XMPPConnection *)sender
        didReceiveEvent: (enum ConnectionEvent) connectionEvent
                 params: (NSDictionary *)params;

/*!
    @brief      Removes an invocation for a specific connection
    @param      connection The connection to remove.
 */
- (void) removeInvocationsForConnection: (NSString *)connection;

/*!
    @brief      Registers an invocation for a servie
    @details    Internal implementation. Should only called from subclasses.
    @param      invocation The invocation called if service name arrives.
    @param      serviceName The service name to register.
 */
- (void) _registerInvocation: (NSInvocation *) invocation forServiceName: (NSString *)serviceName;

/*!
    @brief      Copies all Invocations for a service
    @param      serviceName The service.
    @return     the invocations if found nil otherwise.
 */
- (NSArray *)_copyInvocationsForServiceName: (NSString *)serviceName;

/*!
    @brief      Removes an invocation for a servie
    @details    Internal implementation. Should only called from subclasses.
    @param      aTarget The target for the service. Can be nil.
    @param      serviceName The service name to remove. Can be nil.
 */
- (void) _removeInvocationForTarget: (id)aTarget serviceName: (NSString *)aServiceName;

@end
